import 'package:wowit/controllers/base_controller.dart';
import 'package:wowit/extensions/extensions.dart';

class Filter {
  Filter._();

  static Map<String, String> getFilters(String? filtersString) {
    if (filtersString == null || filtersString.isEmpty) return {};
    Map<String, String> map = {};
    for (var e in filtersString.split(';')) {
      String key = e.split(':').first;
      String value = e.split(':').last;
      if (key.isNotEmpty && value.isNotEmpty && key != value) {
        map[key] = value;
      }
    }
    return map;
  }

  static String toFilterString(Map<String, String?> filters) {
    return filters
        .where(
            (key, value) => key.isNotEmpty && value != null && value.isNotEmpty)
        .entries
        .map((e) => '${e.key}:${e.value}')
        .join(';');
  }
}

mixin FilterMixin on BaseController {
  final Map<String, String?> filters = {};

  /// Thêm [filters] vào [params]
  ///
  /// Ví dụ:
  /// ```dart
  /// @override
  /// Future<Map<String, List<Map<String, Object?>?>?>?>? loadData() async {
  ///   addFilter();
  ///   var data = await SQLService.execute(
  ///     'sp_...',
  ///     params: params,
  ///     formatJson: Parameter.FORMAT_JSON,
  ///   );
  ///   return data;
  /// }
  /// ```
  void addFilter() {
    filters.forEach(
      (key, value) {
        if (key.isNotEmpty && value != null && value.isNotEmpty) {
          params[key] = value;
        }
      },
    );
  }

  /// Áp dụng bộ lọc => cập nhật dữ liệu => cập nhật giao diện
  ///
  /// Hỗ trợ kiểu
  /// ```dart
  /// String?
  /// Map<String, String?>
  /// ```
  ///
  /// Ví dụ
  /// ```dart
  /// applyFilter('idsDanhMuc:1,2,3;idsTinhThanhPho:79;giaMin:200000,giaMax:3000000')
  /// // hoặc
  /// applyFilter({
  ///   'idsDanhMuc': '1,2,3',
  ///   'idsTinhThanhPho': '1',
  ///   'giaMin': '200000',
  ///   'giaMax': '3000000',
  /// })
  /// ```
  void applyFilter(dynamic value) {
    if (value == null) return;

    if (value is String?) {
      filters.clear();
      filters.addAll(Filter.getFilters(value));
      reload();
    } else if (value is Map<String, String?>) {
      filters.clear();
      filters.addAll(value);
      reload();
    }
  }
}
