import 'dart:convert';

import 'package:flutter/foundation.dart';
import 'package:get/get.dart';
import 'package:http/http.dart' as http;
import 'package:wowit/cache_data.dart';
import 'package:wowit/common.dart';
import 'package:wowit/controllers/auth_controller.dart';
import 'package:wowit/methods.dart';
import 'package:wowit/widgets/log_widget/log_controller.dart';

abstract class ISQLService {
  Future<Map<String, List<Map<String, Object?>?>?>?> execute(
    String query, {
    Map<String, Object?>? params,
    bool formatJson = false,
  });
  Future logout();
  Future createAccountIfNotExists(String phoneNumber, String uidPhone);
}

class SQLService {
  const SQLService._();

  static ISQLService? iSQLService;

  static Future createAccountIfNotExists(
      String phoneNumber, String uidPhone) async {
    if (iSQLService != null) {
      await iSQLService?.createAccountIfNotExists(phoneNumber, uidPhone);
    } else {
      await http.post(Methods.getURI('/data/CreateAccount'), body: {
        'phoneNumber': phoneNumber,
        'uidPhone': uidPhone,
      });
    }
  }

  static Future<Map<String, List<Map<String, Object?>?>?>> execute(String query,
      {Map<String, Object?>? params,
      bool formatJson = false,
      bool isCache = false}) async {
    if (iSQLService != null) {
      return await iSQLService?.execute(query,
              params: params, formatJson: formatJson) ??
          {};
    }

    String jparamsString = jsonEncode(
        setupDefaultParameter(query, params, formatJson: formatJson));
    var queryParameters = {
      'query': query,
      'jParam': jparamsString,
    };
    if (kDebugMode) {
      print('Execute query: $query, jParam $jparamsString');
    }
    Map<
        String, // table namme
        List< // list row -> data tabl
            Map<String, Object?>? //data row
            >?> retVal = {};
    try {
      var response =
          await http.post(Methods.getURI('/data/exec'), body: queryParameters);
      if (response.statusCode == 200) {
        if (response.body == '') {
          String log =
              '$query queryParameters $queryParameters response.body is empty';
          LogController.setLog(
            log: log,
            titleLog: '$SQLService.execute',
          );
          if (kDebugMode) {
            print('[EMPTY RESPONSE]: $log');
          }
          return retVal;
        }
        var body = response.body;

        //--> return data set
        Map<String, dynamic> jsonData = json.decode(body);

        jsonData.forEach((key, value) {
          List<dynamic> table = value;
          List<Map<String, Object>> rsTable = [];
          for (int i = 0; i < table.length; i++) {
            rsTable.add(Map<String, Object>.from(table[i]));
          }
          retVal[key] = rsTable;
        });
        if (kDebugMode) {
          print(
            '$SQLService.execute query: $query, result length: ${retVal.length.toString()}',
          );
        }
      } else {
        String log =
            '$SQLService.execute with $query response status code = ${response.statusCode.toString()}';
        LogController.setLog(
          log: log,
          titleLog: '$SQLService.execute',
        );
        if (kDebugMode) {
          print(log);
        }
      }
    } catch (e) {
      String log = '$SQLService.execute $query with exception: $e';
      LogController.setLog(
        log: log,
        titleLog: '$SQLService.execute',
      );
      if (kDebugMode) {
        print(log);
      }
    }

    if (retVal.isNotEmpty && retVal.containsKey('Exception')) {
      var dataException = Methods.getList(retVal, 'Exception');
      if (dataException.isNotEmpty && dataException[0].containsKey('Message')) {
        var exception = Methods.getString(dataException[0], 'Message');
        if (exception != '') {
          if (kDebugMode) {
            print(
                '==================>[Exception]<==================\n==================> Query:$query\n==================> Parameter:$params\n==================> Message:$exception');
          }
        }
      }
    }

    // save cache data
    if (isCache && retVal.isNotEmpty) {
      if (kDebugMode) {
        print('Save cache data');
      }
      CacheData.cache(query, retVal, jParam: params);
    }

    // Get data cache when there is no internet
    if (Get.isRegistered<AuthController>() && isCache) {
      AuthController authController = Get.find();
      if (!authController.isOnline) {
        if (kDebugMode) {
          print('Get data cache');
        }
        return await CacheData.getData(query, jParam: params);
      }
    }
    return retVal;
  }

  static Map<String, Object?> setupDefaultParameter(
    String query,
    Map<String, Object?>? params, {
    bool formatJson = false,
  }) {
    Map<String, Object?> tmp = {};
    tmp.addAll(params ?? {});
    tmp['query'] = query;
    if (!tmp.containsKey('uid') && !tmp.containsKey('@uid')) {
      tmp['uid'] = Common.uid;
    }

    if (kDebugMode) {
      if (!tmp.containsKey('kDebugMode') && !tmp.containsKey('@kDebugMode')) {
        tmp['kDebugMode'] = kDebugMode;
      }
    }
    if (!tmp.containsKey('db') && !tmp.containsKey('@db')) {
      tmp['db'] = Common.idDb;
    }
    if (!tmp.containsKey('languageId') && !tmp.containsKey('@languageId')) {
      tmp['languageId'] = Common.languageId;
    }

    if (!tmp.containsKey('deviceName') && !tmp.containsKey('@deviceName')) {
      tmp['deviceName'] = Common.deviceName;
    }
    if (!tmp.containsKey('@deviceId') && !tmp.containsKey('deviceId')) {
      tmp['deviceId'] = Common.deviceId;
    }

    if (!tmp.containsKey('@mode') && !tmp.containsKey('mode')) {
      tmp['mode'] = 'App';
    }

    if (!tmp.containsKey('@fVersion') &&
        !tmp.containsKey('fVersion') &&
        Common.fVersion > 0) {
      tmp['fVersion'] = Common.fVersion;
    }

    if (!tmp.containsKey('@formatJson') && !tmp.containsKey('formatJson')) {
      tmp['formatJson'] = formatJson;
    }

    return tmp;
  }

  static void logView({
    String query = 'spLogView',
    required int viewID,
    required int dataID,
  }) {
    SQLService.execute(query, params: {
      'userID': Common.uid,
      'idView': viewID,
      'idData': dataID,
    });
  }

  static void logLike({
    String query = 'spLogLike',
    required int idLoai,
    required int idItem,
    required bool isLike,
  }) {
    SQLService.execute(
      query,
      params: {
        'userID': Common.uid,
        'idView': idLoai,
        'idData': idItem,
        'isLike': isLike
      },
    );
  }

  static void logFollow({
    String query = 'spLogFollow',
    required int idLoai,
    required int idItem,
    required bool isFollow,
  }) {
    SQLService.execute(query, params: {
      'userID': Common.uid,
      'idView': idLoai,
      'idData': idItem,
      'isFollow': isFollow
    });
  }
}
